@@ -2,19 +2,19 @@
  * Copyright (C) 2001,2002 Red Hat, Inc.
  * Copyright © 2009, 2010 Christian Persch
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU Library General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 /**
@@ -24,6 +24,8 @@
  *
  * The terminal widget uses these functions to start commands with new controlling
  * pseudo-terminals and to resize pseudo-terminals.
+ *
+ * Since: 0.26
  */
 
 #include <config.h>
@@ -31,8 +33,6 @@
 #include "vtepty.h"
 #include "vtepty-private.h"
 #include "vte.h"
-#include "vteenums.h"
-#include "vtetypebuiltins.h"
 #include "vteversion.h"
 
 #include <sys/types.h>
@@ -64,6 +64,7 @@
 #include <glib.h>
 #include <gio/gio.h>
 #include "debug.h"
+#include "pty.h"
 
 #ifdef MSG_NOSIGNAL
 #define PTY_RECVMSG_FLAGS MSG_NOSIGNAL
@@ -92,10 +93,6 @@ static int _vte_pty_helper_tunnel = -1;
 
 #define VTE_VERSION_NUMERIC ((VTE_MAJOR_VERSION) * 10000 + (VTE_MINOR_VERSION) * 100 + (VTE_MICRO_VERSION))
 
-#if !GLIB_CHECK_VERSION(2, 41, 2)
-#define G_PARAM_EXPLICIT_NOTIFY 0
-#endif
-
 /* Reset the handlers for all known signals to their defaults.  The parent
  * (or one of the libraries it links to) may have changed one to be ignored. */
 static void
@@ -105,7 +102,6 @@ _vte_pty_reset_signal_handlers(void)
 	signal(SIGHUP,  SIG_DFL);
 #endif
 	signal(SIGINT,  SIG_DFL);
-	signal(SIGQUIT, SIG_DFL);
 	signal(SIGILL,  SIG_DFL);
 	signal(SIGABRT, SIG_DFL);
 	signal(SIGFPE,  SIG_DFL);
@@ -215,6 +211,8 @@ typedef struct {
 
 /**
  * VtePty:
+ *
+ * Since: 0.26
  */
 struct _VtePty {
         GObject parent_instance;
@@ -227,6 +225,7 @@ struct _VtePtyPrivate {
         VtePtyFlags flags;
         int pty_fd;
 
+        const char *term;
         VtePtyChildSetupData child_setup_data;
 
         gpointer helper_tag; /* only use when using_helper is TRUE */
@@ -245,6 +244,8 @@ struct _VtePtyClass {
  * @pty: a #VtePty
  *
  * FIXMEchpe
+ *
+ * Since: 0.26
  */
 void
 vte_pty_child_setup (VtePty *pty)
@@ -352,9 +353,9 @@ vte_pty_child_setup (VtePty *pty)
 	_vte_pty_reset_signal_handlers();
 
         /* Now set the TERM environment variable */
-        /* FIXME: Setting environment here seems to have no effect, the merged envp2 will override on exec.
-         * By the way, we'd need to set the one from there, if any. */
-        g_setenv("TERM", VTE_DEFAULT_TERM, TRUE);
+        if (priv->term != NULL) {
+                g_setenv("TERM", priv->term, TRUE);
+        }
 
         g_snprintf (version, sizeof (version), "%u", VTE_VERSION_NUMERIC);
         g_setenv ("VTE_VERSION", version, TRUE);
@@ -369,9 +370,49 @@ vte_pty_child_setup (VtePty *pty)
  * - replace current env rather than adding!
  */
 
+/*
+ * __vte_pty_get_argv:
+ * @command: the command to run
+ * @argv: the argument vector
+ * @flags: (inout) flags from #GSpawnFlags
+ *
+ * Creates an argument vector to pass to g_spawn_async() from @command and
+ * @argv, modifying *@flags if necessary.
+ *
+ * Returns: a newly allocated array of strings. Free with g_strfreev()
+ */
+char **
+__vte_pty_get_argv (const char *command,
+                    char **argv,
+                    GSpawnFlags *flags /* inout */)
+{
+	char **argv2;
+	int i, argc;
+
+        g_return_val_if_fail(command != NULL, NULL);
+
+	/* push the command into argv[0] */
+	argc = argv ? g_strv_length (argv) : 0;
+	argv2 = g_new (char *, argc + 2);
+
+	argv2[0] = g_strdup (command);
+
+	for (i = 0; i < argc; i++) {
+		argv2[i+1] = g_strdup (argv[i]);
+	}
+	argv2[i+1] = NULL;
+
+        if (argv) {
+                *flags |= G_SPAWN_FILE_AND_ARGV_ZERO;
+        }
+
+	return argv2;
+}
+
 /*
  * __vte_pty_merge_environ:
  * @envp: environment vector
+ * @term_value: the value for the TERM env variable, or %NULL
  * @inherit: whether to use the parent environment
  *
  * Merges @envp to the parent environment, and returns a new environment vector.
@@ -380,6 +421,7 @@ vte_pty_child_setup (VtePty *pty)
  */
 static gchar **
 __vte_pty_merge_environ (char **envp,
+                         const char *term_value,
                          gboolean inherit)
 {
 	GHashTable *table;
@@ -401,9 +443,6 @@ __vte_pty_merge_environ (char **envp,
 		g_strfreev (parent_environ);
 	}
 
-        /* Make sure the one in envp overrides the default. */
-        g_hash_table_replace (table, g_strdup ("TERM"), g_strdup (VTE_DEFAULT_TERM));
-
 	if (envp != NULL) {
 		for (i = 0; envp[i] != NULL; i++) {
 			name = g_strdup (envp[i]);
@@ -416,6 +455,9 @@ __vte_pty_merge_environ (char **envp,
 		}
 	}
 
+        if (term_value != NULL)
+                g_hash_table_replace (table, g_strdup ("TERM"), g_strdup (term_value));
+
         g_hash_table_replace (table, g_strdup ("VTE_VERSION"), g_strdup_printf ("%u", VTE_VERSION_NUMERIC));
 
 	array = g_ptr_array_sized_new (g_hash_table_size (table) + 1);
@@ -430,6 +472,34 @@ __vte_pty_merge_environ (char **envp,
 	return (gchar **) g_ptr_array_free (array, FALSE);
 }
 
+/*
+ * __vte_pty_get_pty_flags:
+ * @lastlog: %TRUE if the session should be logged to the lastlog
+ * @utmp: %TRUE if the session should be logged to the utmp/utmpx log
+ * @wtmp: %TRUE if the session should be logged to the wtmp/wtmpx log
+ *
+ * Combines the @lastlog, @utmp, @wtmp arguments into the coresponding
+ * #VtePtyFlags flags.
+ *
+ * Returns: flags from #VtePtyFlags
+ */
+VtePtyFlags
+__vte_pty_get_pty_flags(gboolean lastlog,
+                        gboolean utmp,
+                        gboolean wtmp)
+{
+        VtePtyFlags flags = VTE_PTY_DEFAULT;
+
+        if (!lastlog)
+                flags |= VTE_PTY_NO_LASTLOG;
+        if (!utmp)
+                flags |= VTE_PTY_NO_UTMP;
+        if (!wtmp)
+                flags |= VTE_PTY_NO_WTMP;
+
+        return flags;
+}
+
 /*
  * __vte_pty_spawn:
  * @pty: a #VtePty
@@ -489,7 +559,7 @@ __vte_pty_spawn (VtePty *pty,
         spawn_flags &= ~VTE_SPAWN_NO_PARENT_ENVV;
 
         /* add the given environment to the childs */
-        envp2 = __vte_pty_merge_environ (envv, inherit_envv);
+        envp2 = __vte_pty_merge_environ (envv, pty->priv->term, inherit_envv);
 
         _VTE_DEBUG_IF (VTE_DEBUG_MISC) {
                 g_printerr ("Spawing command:\n");
@@ -541,12 +611,54 @@ __vte_pty_spawn (VtePty *pty,
         return FALSE;
 }
 
+/*
+ * __vte_pty_fork:
+ * @pty: a #VtePty
+ * @pid: (out) a location to store a #GPid, or %NULL
+ * @error: a location to store a #GError, or %NULL
+ *
+ * Forks and calls vte_pty_child_setup() in the child.
+ *
+ * Returns: %TRUE on success, or %FALSE on failure with @error filled in
+ */
+gboolean
+__vte_pty_fork(VtePty *pty,
+               GPid *pid,
+               GError **error)
+{
+#ifdef HAVE_FORK
+        gboolean ret = TRUE;
+
+        *pid = fork();
+        switch (*pid) {
+                case -1:
+                        g_set_error(error,
+                                    G_SPAWN_ERROR,
+                                    G_SPAWN_ERROR_FAILED,
+                                    "Unable to fork: %s",
+                                    g_strerror(errno));
+                        ret = FALSE;
+                case 0: /* child */
+                        vte_pty_child_setup(pty);
+                        break;
+                default: /* parent */
+                        break;
+        }
+
+	return ret;
+#else /* !HAVE_FORK */
+        g_set_error_literal(error, G_SPAWN_ERROR, G_SPAWN_ERROR_FAILED,
+                            "No fork implementation");
+        return FALSE;
+#endif /* HAVE_FORK */
+}
+
 /**
  * vte_pty_set_size:
  * @pty: a #VtePty
  * @rows: the desired number of rows
  * @columns: the desired number of columns
- * @error: (allow-none): return location to store a #GError, or %NULL
+ * @error: (allow-none); return location to store a #GError, or %NULL
  *
  * Attempts to resize the pseudo terminal's window size.  If successful, the
  * OS kernel will send #SIGWINCH to the child process group.
@@ -554,6 +666,8 @@ __vte_pty_spawn (VtePty *pty,
  * If setting the window size failed, @error will be set to a #GIOError.
  *
  * Returns: %TRUE on success, %FALSE on failure with @error filled in
+ *
+ * Since: 0.26
  */
 gboolean
 vte_pty_set_size(VtePty *pty,
@@ -561,12 +675,15 @@ vte_pty_set_size(VtePty *pty,
                  int columns,
                  GError **error)
 {
+        VtePtyPrivate *priv;
 	struct winsize size;
         int master;
 	int ret;
 
         g_return_val_if_fail(VTE_IS_PTY(pty), FALSE);
 
+        priv = pty->priv;
+
         master = vte_pty_get_fd(pty);
 
 	memset(&size, 0, sizeof(size));
@@ -608,6 +725,8 @@ vte_pty_set_size(VtePty *pty,
  * If getting the window size failed, @error will be set to a #GIOError.
  *
  * Returns: %TRUE on success, %FALSE on failure with @error filled in
+ *
+ * Since: 0.26
  */
 gboolean
 vte_pty_get_size(VtePty *pty,
@@ -615,12 +734,15 @@ vte_pty_get_size(VtePty *pty,
                  int *columns,
                  GError **error)
 {
+        VtePtyPrivate *priv;
 	struct winsize size;
         int master;
 	int ret;
 
         g_return_val_if_fail(VTE_IS_PTY(pty), FALSE);
 
+        priv = pty->priv;
+
         master = vte_pty_get_fd(pty);
 
 	memset(&size, 0, sizeof(size));
@@ -1325,6 +1447,8 @@ _vte_pty_open_with_helper(VtePty *pty,
  * discipline do multibyte backspace correctly.
  *
  * Returns: %TRUE on success, %FALSE on failure with @error filled in
+ *
+ * Since: 0.26
  */
 gboolean
 vte_pty_set_utf8(VtePty *pty,
@@ -1339,7 +1463,7 @@ vte_pty_set_utf8(VtePty *pty,
         g_return_val_if_fail(VTE_IS_PTY(pty), FALSE);
 
         priv = pty->priv;
-        g_return_val_if_fail (priv->pty_fd != -1, FALSE);
+        g_return_val_if_fail (priv->pty_fd > 0, FALSE);
 
         if (tcgetattr(priv->pty_fd, &tio) == -1) {
                 int errsv = errno;
@@ -1376,6 +1500,8 @@ vte_pty_set_utf8(VtePty *pty,
  *
  * Cleans up the PTY, specifically any logging performed for the session.
  * The file descriptor to the PTY master remains open.
+ *
+ * Since: 0.26
  */
 void
 vte_pty_close (VtePty *pty)
@@ -1419,6 +1545,7 @@ enum {
         PROP_0,
         PROP_FLAGS,
         PROP_FD,
+        PROP_TERM
 };
 
 /* GInitable impl */
@@ -1517,6 +1644,7 @@ vte_pty_init (VtePty *pty)
         priv->foreign = FALSE;
         priv->using_helper = FALSE;
         priv->helper_tag = NULL;
+        priv->term = vte_terminal_get_default_emulation(NULL /* that's ok, this function is just retarded */); /* already interned */
 }
 
 static void
@@ -1559,6 +1687,10 @@ vte_pty_get_property (GObject    *object,
                 g_value_set_int(value, vte_pty_get_fd(pty));
                 break;
 
+        case PROP_TERM:
+                g_value_set_string(value, priv->term);
+                break;
+
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
         }
@@ -1583,6 +1715,10 @@ vte_pty_set_property (GObject      *object,
                 priv->foreign = (priv->pty_fd != -1);
                 break;
 
+        case PROP_TERM:
+                vte_pty_set_term(pty, g_value_get_string(value));
+                break;
+
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
         }
@@ -1604,6 +1740,8 @@ vte_pty_class_init (VtePtyClass *klass)
          *
          * Controls how the session is recorded in lastlog, utmp, and wtmp,
          * and whether to use the GNOME PTY helper.
+         *
+         * Since: 0.26
          */
         g_object_class_install_property
                 (object_class,
@@ -1613,12 +1751,14 @@ vte_pty_class_init (VtePtyClass *klass)
                                      VTE_PTY_DEFAULT,
                                      G_PARAM_READWRITE |
                                      G_PARAM_CONSTRUCT_ONLY |
-                                     G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY));
+                                     G_PARAM_STATIC_STRINGS));
 
         /**
          * VtePty:fd:
          *
          * The file descriptor of the PTY master.
+         *
+         * Since: 0.26
          */
         g_object_class_install_property
                 (object_class,
@@ -1627,7 +1767,23 @@ vte_pty_class_init (VtePtyClass *klass)
                                    -1, G_MAXINT, -1,
                                    G_PARAM_READWRITE |
                                    G_PARAM_CONSTRUCT_ONLY |
-                                   G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY));
+                                   G_PARAM_STATIC_STRINGS));
+
+        /**
+         * VtePty:term:
+         *
+         * The value to set for the TERM environment variable just after
+         * forking.
+         *
+         * Since: 0.26
+         */
+        g_object_class_install_property
+                (object_class,
+                 PROP_TERM,
+                 g_param_spec_string ("term", NULL, NULL,
+                                      "xterm",
+                                      G_PARAM_READWRITE |
+                                      G_PARAM_STATIC_STRINGS));
 }
 
 /* public API */
@@ -1639,6 +1795,8 @@ vte_pty_class_init (VtePtyClass *klass)
  * enumeration. See #GError for more information on error domains.
  *
  * Returns: the error domain for VTE PTY errors
+ *
+ * Since: 0.26
  */
 GQuark
 vte_pty_error_quark(void)
@@ -1652,9 +1810,8 @@ vte_pty_error_quark(void)
 }
 
 /**
- * vte_pty_new_sync: (constructor)
+ * vte_pty_new:
  * @flags: flags from #VtePtyFlags
- * @cancellable: (allow-none): a #GCancellable, or %NULL
  * @error: (allow-none): return location for a #GError, or %NULL
  *
  * Allocates a new pseudo-terminal.
@@ -1668,7 +1825,7 @@ vte_pty_error_quark(void)
  * vte_pty_child_setup() directly as the child setup function, or call
  * vte_pty_child_setup() from your own child setup function supplied.
  *
- * When using vte_terminal_spawn_sync() with a custom child setup
+ * When using vte_terminal_fork_command_full() with a custom child setup
  * function, vte_pty_child_setup() will be called before the supplied
  * function; you must not call it again.
  *
@@ -1686,23 +1843,23 @@ vte_pty_error_quark(void)
  * be no fallback to allocate a PTY using Unix98 PTY functions.
  *
  * Returns: (transfer full): a new #VtePty, or %NULL on error with @error filled in
+ *
+ * Since: 0.26
  */
 VtePty *
-vte_pty_new_sync (VtePtyFlags flags,
-                  GCancellable *cancellable,
-                  GError **error)
+vte_pty_new (VtePtyFlags flags,
+             GError **error)
 {
         return g_initable_new (VTE_TYPE_PTY,
-                               cancellable,
+                               NULL /* cancellable */,
                                error,
                                "flags", flags,
                                NULL);
 }
 
 /**
- * vte_pty_new_foreign_sync: (constructor)
+ * vte_pty_new_foreign:
  * @fd: (transfer full): a file descriptor to the PTY
- * @cancellable: (allow-none): a #GCancellable, or %NULL
  * @error: (allow-none): return location for a #GError, or %NULL
  *
  * Creates a new #VtePty for the PTY master @fd.
@@ -1713,16 +1870,17 @@ vte_pty_new_sync (VtePtyFlags flags,
  * and close it on finalize.
  *
  * Returns: (transfer full): a new #VtePty for @fd, or %NULL on error with @error filled in
+ *
+ * Since: 0.26
  */
 VtePty *
-vte_pty_new_foreign_sync (int fd,
-                          GCancellable *cancellable,
-                          GError **error)
+vte_pty_new_foreign (int fd,
+                     GError **error)
 {
         g_return_val_if_fail(fd >= 0, NULL);
 
         return g_initable_new (VTE_TYPE_PTY,
-                               cancellable,
+                               NULL /* cancellable */,
                                error,
                                "fd", fd,
                                NULL);
@@ -1747,3 +1905,243 @@ vte_pty_get_fd (VtePty *pty)
 
         return priv->pty_fd;
 }
+
+/**
+ * vte_pty_set_term:
+ * @pty: a #VtePty
+ * @emulation: (allow-none): the name of a terminal description, or %NULL
+ *
+ * Sets what value of the TERM environment variable to set just after forking.
+ *
+ * Since: 0.26
+ */
+void
+vte_pty_set_term (VtePty *pty,
+                  const char *emulation)
+{
+        VtePtyPrivate *priv;
+
+        g_return_if_fail(VTE_IS_PTY(pty));
+        g_return_if_fail(emulation != NULL);
+
+        priv = pty->priv;
+        emulation = g_intern_string(emulation);
+        if (emulation == priv->term)
+                return;
+
+        priv->term = emulation;
+        g_object_notify(G_OBJECT(pty), "term");
+}
+
+/* Reimplementation of the ugly deprecated APIs _vte_pty_*() */
+
+#ifndef VTE_DISABLE_DEPRECATED_SOURCE
+
+static GHashTable *fd_to_pty_hash = NULL;
+
+static VtePty *
+get_vte_pty_for_fd (int fd)
+{
+        VtePty *pty;
+
+        if (fd_to_pty_hash != NULL &&
+            (pty = g_hash_table_lookup(fd_to_pty_hash, &fd)) != NULL)
+                return pty;
+
+        g_warning("No VtePty found for fd %d!\n", fd);
+        return NULL;
+}
+
+/**
+ * _vte_pty_open:
+ * @child: location to store the new process's ID
+ * @env_add: a list of environment variables to add to the child's environment
+ * @command: name of the binary to run
+ * @argv: arguments to pass to @command
+ * @directory: directory to start the new command in, or %NULL
+ * @columns: desired window columns
+ * @rows: desired window rows
+ * @lastlog: %TRUE if the lastlog should be updated
+ * @utmp: %TRUE if the utmp or utmpx log should be updated
+ * @wtmp: %TRUE if the wtmp or wtmpx log should be updated
+ *
+ * Starts a new copy of @command running under a psuedo-terminal, optionally in
+ * the supplied @directory, with window size set to @rows x @columns
+ * and variables in @env_add added to its environment.  If any combination of
+ * @lastlog, @utmp, and @wtmp is set, then the session is logged in the
+ * corresponding system files.
+ *
+ * Returns: an open file descriptor for the pty master, -1 on failure
+ *
+ * Deprecated: 0.26: Use #VtePty together with fork() or the g_spawn_async() family of functions instead
+ */
+int
+_vte_pty_open(pid_t *child,
+              char **env_add,
+              const char *command,
+              char **argv,
+              const char *directory,
+              int columns,
+              int rows,
+              gboolean lastlog,
+              gboolean utmp,
+              gboolean wtmp)
+{
+        VtePty *pty;
+        GPid pid;
+        gboolean ret;
+
+        pty = vte_pty_new(__vte_pty_get_pty_flags (lastlog, utmp, wtmp), NULL);
+        if (pty == NULL)
+                return -1;
+
+        if (command != NULL) {
+                char **real_argv;
+                GSpawnFlags spawn_flags;
+
+                spawn_flags = G_SPAWN_CHILD_INHERITS_STDIN |
+                              G_SPAWN_SEARCH_PATH;
+                real_argv = __vte_pty_get_argv(command, argv, &spawn_flags);
+                ret = __vte_pty_spawn(pty,
+                                      directory,
+                                      real_argv,
+                                      env_add,
+                                      spawn_flags,
+                                      NULL, NULL,
+                                      &pid,
+                                      NULL);
+                g_strfreev(real_argv);
+        } else {
+                ret = __vte_pty_fork(pty, &pid, NULL);
+        }
+
+        if (!ret) {
+                g_object_unref(pty);
+                return -1;
+        }
+
+        vte_pty_set_size(pty, rows, columns, NULL);
+
+        /* Stash the pty in the hash so we can later retrieve it by FD */
+        if (fd_to_pty_hash == NULL) {
+                fd_to_pty_hash = g_hash_table_new_full(g_int_hash,
+                                                       g_int_equal,
+                                                       NULL,
+                                                       (GDestroyNotify) g_object_unref);
+        }
+
+        g_hash_table_insert(fd_to_pty_hash, &pty->priv->pty_fd, pty /* adopt refcount */);
+
+        if (child)
+                *child = (pid_t) pid;
+
+        return vte_pty_get_fd(pty);
+}
+
+/**
+ * _vte_pty_get_size:
+ * @master: the file descriptor of the PTY master
+ * @columns: a place to store the number of columns
+ * @rows: a place to store the number of rows
+ *
+ * Attempts to read the pseudo terminal's window size.
+ *
+ * Returns: 0 on success, -1 on failure.
+ *
+ * Deprecated: 0.26: Use #VtePty and vte_pty_get_size() instead
+ */
+int
+_vte_pty_get_size(int master,
+                  int *columns,
+                  int *rows)
+{
+        VtePty *pty;
+
+        if ((pty = get_vte_pty_for_fd(master)) == NULL)
+                return -1;
+
+        if (vte_pty_get_size(pty, rows, columns, NULL))
+                return 0;
+
+        return -1;
+}
+
+/**
+ * _vte_pty_set_size:
+ * @master: the file descriptor of the PTY master
+ * @columns: the desired number of columns
+ * @rows: the desired number of rows
+ *
+ * Attempts to resize the pseudo terminal's window size.  If successful, the
+ * OS kernel will send #SIGWINCH to the child process group.
+ *
+ * Returns: 0 on success, -1 on failure.
+ *
+ * Deprecated: 0.26: Use #VtePty and vte_pty_set_size() instead
+ */
+int
+_vte_pty_set_size(int master,
+                  int columns,
+                  int rows)
+{
+        VtePty *pty;
+
+        if ((pty = get_vte_pty_for_fd(master)) == NULL)
+                return -1;
+
+        if (vte_pty_set_size(pty, rows, columns, NULL))
+                return 0;
+
+        return -1;
+}
+
+/**
+ * _vte_pty_set_utf8:
+ * @pty: The pty master descriptor.
+ * @utf8: Whether or not the pty is in UTF-8 mode.
+ *
+ * Tells the kernel whether the terminal is UTF-8 or not, in case it can make
+ * use of the info.  Linux 2.6.5 or so defines IUTF8 to make the line
+ * discipline do multibyte backspace correctly.
+ *
+ * Deprecated: 0.26: Use #VtePty and vte_pty_set_utf8() instead
+ */
+void _vte_pty_set_utf8(int master,
+                       gboolean utf8)
+{
+        VtePty *pty;
+
+        if ((pty = get_vte_pty_for_fd(master)) == NULL)
+                return;
+
+        vte_pty_set_utf8(pty, utf8, NULL);
+}
+
+/**
+ * _vte_pty_close:
+ * @pty: the pty master descriptor.
+ *
+ * Cleans up the PTY associated with the descriptor, specifically any logging
+ * performed for the session.  The descriptor itself remains open.
+ *
+ * Deprecated: 0.26: Use #VtePty and vte_pty_close() instead
+ */
+void _vte_pty_close(int master)
+{
+        VtePty *pty;
+
+        if ((pty = get_vte_pty_for_fd(master)) == NULL)
+                return;
+
+        /* Prevent closing the FD */
+        pty->priv->pty_fd = -1;
+
+        g_hash_table_remove(fd_to_pty_hash, &master);
+
+        if (g_hash_table_size(fd_to_pty_hash) == 0) {
+                g_hash_table_destroy(fd_to_pty_hash);
+                fd_to_pty_hash = NULL;
+        }
+}
+
+#endif /* !VTE_DISABLE_DEPRECATED_SOURCE */
