@@ -227,14 +227,19 @@ struct _VteTerminalPrivate {
 		long insert_delta;	/* insertion offset */
 		VteCell defaults;	/* default characteristics
 					   for insertion of any new
-					   characters */
+					   characters; colors not yet
+					   adjusted for bold/dim/standout */
 		VteCell color_defaults;	/* original defaults
 					   plus the current
-					   fore/back */
+					   fore/back already adjusted for
+					   bold/dim/standout */
 		VteCell fill_defaults;	/* original defaults
 					   plus the current
-					   fore/back with no
+					   fore/back already adjusted for
+					   bold/dim/standout with no
 					   character data */
+		gboolean fg_sgr_extended, bg_sgr_extended;	/* whether fg/bg were set by
+								   256 color sequences */
 		gboolean alternate_charset;
 		gboolean status_line;
 		GString *status_line_contents;
@@ -279,6 +284,7 @@ struct _VteTerminalPrivate {
 	gboolean scroll_background;
 	gboolean scroll_on_output;
 	gboolean scroll_on_keystroke;
+	gboolean alternate_screen_scroll;
 	long scrollback_lines;
 
 	/* Cursor shape */
@@ -305,6 +311,7 @@ struct _VteTerminalPrivate {
 	long mouse_last_x, mouse_last_y;
 	gboolean mouse_autohide;
 	guint mouse_autoscroll_tag;
+	gboolean mouse_xterm_extension;
 	gboolean mouse_urxvt_extension;
 
 	/* State variables for handling match checks. */
@@ -436,6 +443,9 @@ void _vte_terminal_cleanup_tab_fragments_at_cursor (VteTerminal *terminal);
 void _vte_terminal_audible_beep(VteTerminal *terminal);
 void _vte_terminal_visible_beep(VteTerminal *terminal);
 void _vte_terminal_beep(VteTerminal *terminal);
+void _vte_terminal_set_color_internal(VteTerminal *terminal,
+                                      int idx,
+                                      const GdkColor *color);
 
 void _vte_terminal_inline_error_message(VteTerminal *terminal, const char *format, ...) G_GNUC_PRINTF(2,3);
 
