@@ -247,7 +247,7 @@ static const SECTION union { const guint8 data[1900]; const double alignment; vo
   0x04, 0x00, 0x00, 0x00
 } };
 
-static GStaticResource static_resource = { vte_resource_data.data, sizeof (vte_resource_data.data) };
+static GStaticResource static_resource = { vte_resource_data.data, sizeof (vte_resource_data.data), NULL, NULL, NULL };
 extern GResource *vte_get_resource (void);
 GResource *vte_get_resource (void)
 {
